@@ -57,7 +57,7 @@ RSA_set0_key(RSA *rsa, BIGNUM *n, BIGNUM *e, BIGNUM *d)
   rsa->n = n;
   rsa->e = e;
   rsa->d = d;
-  return 0;
+  return 1;
 }
 #endif
 #ifndef HAVE_RSA_GET0_KEY
@@ -114,51 +114,6 @@ const int num_random_bytes = 10240; // 10*8192 bits.
 const char* env_log_stderr = "SIMPLE_TPM_PK11_LOG_STDERR";
 const TSS_UUID srk_uuid = TSS_UUID_SRK;
 
-BEGIN_NAMESPACE();
-template<typename T, T*(*New)(), void(*Free)(T*)>
-class AutoFree {
- public:
-  AutoFree(): resource_(New()) {}
-  AutoFree(T* r): resource_(r) {}
-  AutoFree(const AutoFree&) = delete;
-  AutoFree(const AutoFree&&) = delete;
-  AutoFree& operator=(const AutoFree&) = delete;
-  AutoFree& operator=(const AutoFree&&) = delete;
-  ~AutoFree()
-  {
-    if (!resource_) {
-      return;
-    }
-    try {
-      Free(resource_);
-      resource_ = nullptr;
-    } catch (const std::exception& e) {
-      std::clog << "Exception thrown in free() code.\n";
-      throw;
-    }
-  }
-  T* get() const
-  {
-    return resource_;
-  }
-  T* operator->() const
-  {
-    return resource_;
-  }
-  T* release()
-  {
-    T* ret = resource_;
-    resource_ = nullptr;
-    return ret;
-  }
- private:
-  T* resource_;
-};
-
-typedef AutoFree<RSA, RSA_new, RSA_free> RSAWrap;
-typedef AutoFree<BIGNUM, BN_new, BN_free> BIGNUMWrap;
-END_NAMESPACE();
-
 std::string
 xgetpass(const std::string& prompt)
 {
@@ -242,7 +197,7 @@ TSPIException::code_to_extra(int code)
       "  with srk_pin in the configuration file for the PKCS#11 module.";
   case TSS_LAYER_TSP | TSS_E_COMM_FAILURE:
     return "Likely problem:\n"
-      "  The tscd daemon is not running and listening on TCP port 30003, or there\n"
+      "  The tcsd daemon is not running and listening on TCP port 30003, or there\n"
       "  is a firewall preventing connections to it.\n"
       "Possible solution:\n"
       "  Make sure trousers is started (/etc/init.d/trousers start) correctly, and\n"
@@ -328,7 +283,7 @@ xsprintf(const char* fmt, ...) {
   vsnprintf(&buf[0], s, fmt, va2);
   va_end(va2);
 
-  return std::string(buf.begin(), buf.end());
+  return std::string(buf.begin(), buf.end()-1);
 }
 
 bool
@@ -803,8 +758,8 @@ public_decrypt(const Key& key, const std::string& sig)
 {
   // Load key.
   RSAWrap rsa;
-  if (RSA_set0_key(rsa.get(), string2bn(key.modulus), string2bn(key.exponent),
-                   NULL)) {
+  if (!RSA_set0_key(rsa.get(), string2bn(key.modulus), string2bn(key.exponent),
+                    NULL)) {
     throw std::runtime_error("RSA_set0_key failed");
   }
 
