@@ -19,6 +19,7 @@
 #include<sstream>
 #include<stdexcept>
 #include<string>
+#include<vector>
 
 #include<opencryptoki/pkcs11.h>
 
@@ -48,12 +49,29 @@ public:
   std::string key_pin_;
   bool debug_;
 
-  void debug_log(const char*,...);
+  void debug_log(const char*,...) const;
 
  private:
   void read_file(std::ifstream&);
 };
 
+/*
+typedef struct CK_ATTRIBUTE {
+  CK_ATTRIBUTE_TYPE type;
+  CK_VOID_PTR pValue;
+  CK_ULONG ulValueLen;
+} CK_ATTRIBUTE;
+*/
+
+// deep copy of CK_ATTRIBUTE
+// data_ is raw data (need to be reinterpret_cast to whatever makes sense)
+class CK_ATTRIBUTE_FULL {
+public:
+  CK_ATTRIBUTE_FULL(CK_ATTRIBUTE); // construct from CK_ATTRIBUTE
+  CK_ATTRIBUTE_TYPE type_;
+  std::vector<char> data_;
+};
+
 class Session {
 public:
   Session(const Config&);
@@ -73,7 +91,11 @@ public:
 private:
   Config config_;
   std::string pin_;
-  int findpos_;
+
+  // Set up by FindObjectsInit() used as state between FindObjects()
+  // calls.
+  int              findpos_ = 0;
+  std::vector<CK_ATTRIBUTE_FULL> find_filters_;
 };
 #endif
 /* ---- Emacs Variables ----
